//	Jacobi.cc
//		Solve a system of equations via Jacobi iteration
//
//	g++ -W -Wno-deprecated -Wall -pg Jacobi.cc matvec.cc
//	./a.out
//	gprof -b ./a.out > out
//
#include <cstdlib>            // exit
#include <iostream>
#include <cassert>
using namespace std;
#include "matvec3.hpp"

void JacobiSolve(const int n, const double a[], const double f[], double v[]);

int main()
{
// const char matrixfilename[] = "matrix1.in";
// const char    rhsfilename[] = "rhs1.in";
 const char matrixfilename[] = "matrix2.in";
 const char    rhsfilename[] = "rhs2.in";

 double *matrix, *ud, *f;
    int nrow, ncol, n;

 ConstructMatrix(matrixfilename, nrow, ncol, matrix);

 assert(nrow == ncol);	// Square matrix ??

 ConstructVector(rhsfilename, n, f);

 assert(n == ncol);	// Compatible dimensions ??

 ud = new double [2*n];	// Allocate solution & defect vector
 JacobiSolve(n, matrix, f, ud);
 PrintVec2File(n,ud,"sol.txt");

 delete [] ud;
 delete [] f;
 delete [] matrix;
 return 0;
}

//----------------------------------------------------------------
//	Solve A*u = f  via Jacobi iteration
//		A is square matrix

void JacobiSolve(const int n, const double a[], const double f[], double vd[])
{
 const double EPS=1e-12;
       double *diag;
       double err0, err;
          int iter, i;

 diag = new double [n];		// Allocate memory for aux.vector

 GetMatrixDiag(n,n,a,diag);	// Get Diagonal

 for (i=0; i<n; i++)
 {
     vd[2*i]   = 0.0;		// Initialize solution u
     vd[2*i+1] = f[i];		// Initialize defect d
 }

 err = err0 = L2norm(n,vd,1);	// Initialize error variable
 iter = 0;

 while (err > EPS*err0)
 {
  iter++;
  for (i=0; i<n; i++)		// Update solution
   {
    vd[2*i] += vd[2*i+1]/diag[i];
   }
//  GetDefect_fast2(n, n, a, f, vd); 	// Update defect
//  err = L2norm(n,vd,1);			// Update error variable
  GetDefect_fast2_norm(n, n, a, f, vd, err); 	// Update defect
//  if (iter==428) exit(0);
 }

 cout << iter << " iterations for epsilon = " << EPS << endl;

 delete [] diag;

 return;
}
