//	bung 5 - Aufgabe A/B/C
//	g++ myvector.cc

#include <iostream.h>
#include <fstream.h>
#include <math.h>
//			Include my own declarations
#include "myclass.hh"

//			I have to define the declared functions

// ---------------------------------------------------------------------

MyVectorClass :: MyVectorClass()
{
 length = 0;
 u = 0;
 kmax = -1;
 l2norm = amax= -1.;
}
// -------------------------------------------------------------

MyVectorClass :: MyVectorClass(const int n, const int freq)
{
 int   i;
 double h;
 
 if (n <= 0)
  {
    cout << "WARNING: No memory allocated  n = " << n << endl;
    length = 0;
    u = 0;
    kmax   = -1; 
    amax   = -1.0;
    l2norm = -1.0;
  }
 else
  {
    length = n;
    u = new double[length];
    
    h = 1.0/(length-1);
    for (i=0; i<length; i++)
     {
       u[i] = sin(freq*M_PI*i*h);
     }
    
    l2norm = L2Norm(length,u);
    MaxPos(length, u, kmax, amax);
  }
 
 return;
}
// -------------------------------------------------------------

MyVectorClass :: MyVectorClass(const char filename[])
{
 int   i;
 ifstream infile(filename);
 
 infile >> length;		// read dimension
 
 if (length <= 0)
  {
    cout << "WARNING: No memory allocated  length = " << length << endl;
    length = 0;
    u = 0;
   kmax   = -1; 
    amax   = -1.0;
    l2norm = -1.0;
  }
 else
  {
    u = new double[length];
    
    for (i=0; i<length; i++)
     {
       infile >> u[i];
     }
    
    l2norm = L2Norm(length,u);
    MaxPos(length, u, kmax, amax);
  }
 
 return;
}
// -------------------------------------------------------------

MyVectorClass :: MyVectorClass(const MyVectorClass& orig)
{
 int i;
 
 kmax   = orig.kmax;
 amax   = orig.amax;
 l2norm = orig.l2norm;
 length = orig.length;
 
 if (length <= 0)
  {
    u = 0;
  }
 else
  {
    u = new double[length];
    for (i=0; i<length; i++)
     {
       u[i] = orig.u[i];
     }
  }
 
 return;
}

// -------------------------------------------------------------

MyVectorClass :: ~MyVectorClass()
{
 if (u != 0) 
  {
   delete [] u;
  }
  
 return;
}

// -------------------------------------------------------------
//	Copy constructor

MyVectorClass&  MyVectorClass:: operator=(const MyVectorClass& orig)
{
 int i;
 
 if (u != 0)  delete [] u;
 
 kmax   = orig.kmax;
 amax   = orig.amax;
 l2norm = orig.l2norm;
 length = orig.length;
 
 if (length <= 0)
  {
    u = 0;
  }
 else
  {
    u = new double[length];
    for (i=0; i<length; i++)
     {
       u[i] = orig.u[i];
     }
  }

 return *this;
}

// -------------------------------------------------------------
//	Append the vector
//
void MyVectorClass :: Append(const int m) 
{
 int    i,n;
 double *pt;		// temporary pointer
 
 if (m <= 0)
  {
    cout << "WARNING: No element appended  m = " << m << endl;
  }
 else
  {
    n = length; 
//		Allocate new memory range
    pt = new double [n+m];

//		Copy existing data from old (short) vector    
    for (i=0; i<n; i++)
      {
        pt[i] = u[i]; 
      }
    for (i=n; i<n+m; i++)
      {
        pt[i] = 0.0;	// doesn't change norm an absolute max.
      }
//		Free old memory range
    delete [] u;

//		Set pointer to newly allocated memory
//			(don't free that memory)
    u = pt;
//		Set new vector length
    length = n+m;
  } 
 return;
}
// -------------------------------------------------------------
//	Print data of MyVectorClass

void MyVectorClass :: Print() const
{
 int i;
 const int &n  = length;	// Reference on orig.length
 
 cout << endl;
 cout << " Vector length = " << n << endl;
 if (n <= 15)
  {
   for (i=0; i<n; i++)
    {
     cout << "          " << i << " : " << u[i] << endl;
    }
  }
 cout << "       L2-norm = " << l2norm << endl;
 cout << "       abs.Max = " << amax << "  at position " << kmax << endl;
 cout << endl;
 
 return;
}

// -------------------------------------------------------------
double MyVectorClass :: InnerProd(const MyVectorClass & right) const
{
 double s;
 int    i;
 
 s = 0.0;
 for (i=0; i<length; i++)
  {
   s += u[i]*right.u[i];
  }
 
 return s;
}

// -------------------------------------------------------------
//	Calculate L2-Norm of a vector
//		(case n<=0 is included)

double L2Norm(const int n, const double v[])
{
 double t;
 int    i;
  
 t = 0.0;
 for ( i=0; i<n; i++)
  { 
   t += v[i]*v[i]; 
  } 
 
 return sqrt(t);
}

// -------------------------------------------------------------
//	Determine value and position of the absolute max in a vector
//		(case n<=0 is included)

void MaxPos(const int n, const double v[], int& kmax, double& amax)
{
 double vi;
 int     i;

 kmax = -1;
 amax = -1.0;

 for ( i=0; i<n; i++)
  {
   vi = fabs(v[i]);
   if ( vi > amax )
    {
      amax = vi;
      kmax = i;
    }
  } 
  
 return;
}























