//	bung 5 - Aufgabe A/B/C
//	g++ myvector.cc

#include <iostream.h>
#include <math.h>
//			Include my own declarations
#include "myvector.hh"

//			I have to define the declared functions

// -------------------------------------------------------------
//	Allocate memory

void Define(MyVector& mv, const int n)
{
 mv.kmax   = -1; 		// Initialize data
 mv.amax   = -1.0;
 mv.l2norm = 0.0;
 
 if (n <= 0)
  {
    cout << "WARNING: No memory allocated  n = " << n << endl;
    mv.length = 0;
    mv.u = 0;
  }
 else
  {
    mv.length = n;
    mv.u = new double[mv.length];
  }
 
 return;
}

// -------------------------------------------------------------
//	Deallocate Memory

void Remove(MyVector& mv)
{
 if (mv.u != 0) 
  {
   delete [] mv.u;
  }
  
 return;
}

// -------------------------------------------------------------
//	Copy one MyVector to another

void Copy(MyVector& left, const MyVector& right)
{
 int i;
 
 left.length = right.length;
 left.kmax   = right.kmax;
 left.l2norm = right.l2norm; 
 left.amax   = right.amax;
//			Allocate the memory for left
 left.u = new double[left.length];    

//			Copy u_vec auf w_vec
 for ( i=0; i<left.length; i++)
  {
   left.u[i] = right.u[i]; 
  }

 return;
}

// -------------------------------------------------------------
//	Initialize date in Structure

void Init(MyVector& mv, const int k)
{
 int i;
 int     &n  = mv.length;	// Reference on mv.length
 double* &ru = mv.u;		// Reference on array mv.u
 double  h   = 1.0/(n-1);
 
 for (i=0; i<n; i++)
  {
   ru[i] = sin(k*M_PI*i*h);
  }

//	Calculate L2-norm
 mv.l2norm = L2Norm(mv.length, mv.u); // the same as  L2Norm(n, ru)

//	Determine max. and its position
 MaxPos(n, ru, mv.kmax, mv.amax);

 return;
}

// -------------------------------------------------------------
//	Print data in MyVector

void Print(const MyVector& mv)
{
 int i;
 const int &n  = mv.length;	// Reference on mv.length
 
 cout << endl;
 cout << " Vector length = " << n << endl;
 if (n <= 15)
  {
   for (i=0; i<n; i++)
    {
     cout << "          " << i << " : " <<  mv.u[i] << endl;
    }
  }
 cout << "       L2-norm = " << mv.l2norm << endl;
 cout << "       abs.Max = " << mv.amax << "  at position " << mv.kmax << endl;
 cout << endl;
 
 return;
}

// -------------------------------------------------------------
//	Calculate L2-Norm of a vector
//		(case n<=0 is included)

double L2Norm(const int n, const double v[])
{
 double t;
 int    i;
  
 t = 0.0;
 for ( i=0; i<n; i++)
  { 
   t += v[i]*v[i]; 
  } 
 
 return sqrt(t);
}

// -------------------------------------------------------------
//	Determine value and position of the absolute max in a vector
//		(case n<=0 is included)

void MaxPos(const int n, const double v[], int& kmax, double& amax)
{
 double vi;
 int     i;

 kmax = -1;
 amax = -1.0;

 for ( i=0; i<n; i++)
  {
   vi = fabs(v[i]);
   if ( vi > amax )
    {
      amax = vi;
      kmax = i;
    }
  } 
  
 return;
}


// -------------------------------------------------------------
//	Append the vector
//
void Append(MyVector& mv, const int m)
{
 int    i,n;
 double *pt;		// temporary pointer
 
 if (m <= 0)
  {
    cout << "WARNING: No element appended  m = " << m << endl;
  }
 else
  {
    n = mv.length; 
//		Allocate new memory range
    pt = new double [n+m];

//		Copy existing data from old (short) vector    
    for (i=0; i<n; i++)
      {
        pt[i] = mv.u[i]; 
      }
    for (i=n; i<n+m; i++)
      {
        pt[i] = 0.0;	// doesn't change norm an absolute max.
      }
//		Free old memory range
    delete [] mv.u;

//		Set pointer to newly allocated memory
//			(don't free that memory)
    mv.u = pt;
//		Set new vector length
    mv.length = n+m;
  } 
 return;
}





















